#pragma once

#define TEST_BITS_LENGTH 1024

#include "modules/common/sathelper/packetfixer.h"
#include "modules/common/viterbi/cc_decoder_impl.h"
#include "modules/common/viterbi/cc_encoder_impl.h"
#include "modules/common/viterbi/depuncture_bb_impl.h"
#include "modules/common/repack_bits_byte.h"

namespace fengyun
{
    class NewFengyunAHRPTViterbi
    {
    private:
        // Settings
        const float d_ber_thresold;
        const int d_outsync_after;
        const int d_buffer_size;

        // Variables
        int d_outsinc;
        int d_state;
        bool d_first;
        float d_bers[2][2];
        float d_ber;

        // BER Decoders
        fec::depuncture_bb_impl depunc_ber;
        fec::code::cc_decoder_impl cc_decoder_in_ber;
        fec::code::cc_encoder_impl cc_encoder_in_ber;

        // BER Buffers
        uint8_t d_ber_test_buffer[TEST_BITS_LENGTH];
        uint8_t d_ber_input_buffer[TEST_BITS_LENGTH];
        uint8_t d_ber_input_buffer_depunc[TEST_BITS_LENGTH * 2]; // 1.5
        uint8_t d_ber_decoded_buffer[TEST_BITS_LENGTH * 2];
        uint8_t d_ber_encoded_buffer[TEST_BITS_LENGTH * 2]; // 1.5

        // Current phase status
        sathelper::PhaseShift d_phase_shift;
        bool d_iq_inv;
        sathelper::PacketFixer phaseShifter;

        // Work buffers
        uint8_t *fixed_soft_packet;
        uint8_t *converted_buffer;
        uint8_t *depunc_buffer;
        uint8_t *output_buffer;

        // Main decoder
        fec::code::cc_decoder_impl cc_decoder_in;
        fec::depuncture_bb_impl depunc;

        // Repacker
        RepackBitsByte repacker;

        float getBER(uint8_t *input);

    public:
        NewFengyunAHRPTViterbi(float ber_threshold, int outsync_after, int buffer_size);
        ~NewFengyunAHRPTViterbi();

        int work(uint8_t *input, int size, uint8_t *output);
        float ber();
        int getState();
    };
} // namespace npp